#include <stdlib.h>
#include "aswm.h"
#include "event_handlers/create.h"
#include "event_handlers/configure.h"
#include "event_handlers/map.h"
#include "event_handlers/reparent.h"
#include "event_handlers/destroy.h"

Aswm _aswm;

void aswm_open() {
	_aswm.display = XOpenDisplay(NULL);
	_aswm.root_window = DefaultRootWindow(_aswm.display);


	_aswm.root_tile = aswm_alloc_root_tile(_aswm.display, _aswm.root_window);

	XSelectInput(_aswm.display, _aswm.root_window, SubstructureRedirectMask | SubstructureNotifyMask);
	XSync(_aswm.display, 0);

	atexit(aswm_close);
}

void aswm_event_loop() {
	for (;;) {
		XEvent e;
		XNextEvent(_aswm.display, &e);

		switch(e.type) {
			case CreateNotify:
				OnCreateNotify(&e.xcreatewindow);
				break;
			case ConfigureRequest:
				OnConfigureRequest(&e.xconfigurerequest);
				break;
			case MapRequest:
				OnMapRequest(&_aswm, &e.xmaprequest);
				break;
			case DestroyNotify:
				OnDestroyNotify(&e.xdestroywindow);
				break;
			case ReparentNotify:
				OnReparentNotify(&e.xreparent);
				break;
			default:
				break;
					/* LOG(WARNING) << "Ignored event"; */
		}
	}
}

void aswm_close() {
	aswm_free_tile(_aswm.root_tile);
	XCloseDisplay(_aswm.display);
}

int main(int argc, char** argv) {
	aswm_open();
	aswm_event_loop();
	aswm_close();
}
